<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xmleye="http://www.uca.es/xmleye"
  xmlns:ppacl2="http://www.uca.es/ppACL2"
  xmlns:util="http://www.uca.es/util">

<xsl:import href="preproc_xml"/>

<!-- 
    Resumen:
    * se mueve la información del método usado a la submeta/subfórmula
    padre.
    * se añaden iconos de éxito/fracaso y etiquetas de
    submeta/subfórmula.
    * se marcan aquellos nodos que corresponden con eventos ACL2. (@isevent=1)
    * se crean enlaces entre las reglas usadas dentro de una
    demostración y sus definiciones, si son funciones del mismo fuente .lisp.
-->

<!-- No queremos que las inducciones aparezcan tras la meta principal -->
<!-- (Goal), sino dentro de ella. Así, defthm/defun/thm no deben -->
<!-- llamar a las fórmulas, sino el elemento subgoal hermano. -->
<xsl:template match="*[self::defthm or self::defun or self::thm]" mode="process-children">
  <xsl:apply-templates select="*[not(self::formula)]" mode="process-node"/>
</xsl:template>

<!-- Movemos los atributos y descendientes el nodo del método -->
<!-- utilizado en la submeta al padre, menos en los elementos que son -->
<!-- sólo una salida (no hay método).  -->
<xsl:template match="*[self::subgoal]" mode="process-children">
   <xsl:apply-templates select="output" mode="process-node"/>
   <xsl:apply-templates select="*[not(self::output)]/*" mode="process-node"/>
   <xsl:apply-templates select="following-sibling::formula" mode="process-node"/>
</xsl:template>

<!-- RESTO DE REGLAS -->

<!-- Eventos -->

<xsl:template match="defun|defthm|thm">
  <xsl:if test="@name">
    <xsl:attribute name="nodelabel">
      <xsl:value-of select="concat(name(.), ' ', @name)"/>
    </xsl:attribute>
  </xsl:if>
  <xsl:attribute name="isevent">1</xsl:attribute>
  <xsl:call-template name="attach-success-icon"/>
  <xsl:if test="@name">
    <xsl:call-template name="search-uses"/>
  </xsl:if>
</xsl:template>

<xsl:template match="encapsulate|local">
   <xsl:attribute name="isevent">1</xsl:attribute>
</xsl:template>

<xsl:template match="*[@name and not(ancestor::*[@name])]" priority="-1">
   <xsl:attribute name="nodelabel">
     <xsl:value-of select="concat(local-name(.), ' ', @name)"/>
   </xsl:attribute>
   <xsl:attribute name="isevent">1</xsl:attribute>
</xsl:template>

<xsl:template match="certifybook">
  <xsl:attribute name="nodelabel">
    <xsl:value-of select="concat($ppacl2:certify, @name)"/>
  </xsl:attribute>
  <xsl:attribute name="isevent">1</xsl:attribute>
</xsl:template>

<!-- <unknown> elements should not be hidden -->

<xsl:template match="unknown"/>

<!-- Submetas/Subfórmulas -->

<xsl:template match="subgoal">
  <xsl:attribute name="nodelabel">
    <xsl:value-of select="@label"/>
  </xsl:attribute>

  <xsl:if test="descendant::failure">
    <xsl:attribute name="nodeicon">
     <xsl:value-of select="$xmleye:failure-icon"/>
    </xsl:attribute>
  </xsl:if>

  <xsl:choose>
   <xsl:when test="not(output[1])">
    <xsl:attribute name="method">
      <xsl:value-of select="name(*[1])"/>
    </xsl:attribute>
    <xsl:copy-of select="*[1]/@*"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:apply-imports/>
   </xsl:otherwise>
  </xsl:choose>

</xsl:template>

<!-- Partes de un CERTIFY-BOOK -->

<xsl:template match="checksum">
  <xsl:attribute name="nodelabel">
    <xsl:value-of select="$ppacl2:checksum"/>
  </xsl:attribute>
</xsl:template>

<xsl:template match="compilation">
  <xsl:attribute name="nodelabel">
    <xsl:value-of select="$ppacl2:compilation"/>
  </xsl:attribute>
</xsl:template>

<xsl:template match="includechk">
  <xsl:attribute name="nodelabel">
    <xsl:value-of select="$ppacl2:includechk"/>
  </xsl:attribute>
</xsl:template>

<xsl:template match="verification">
  <xsl:attribute name="nodelabel">
    <xsl:value-of select="$ppacl2:verification"/>
  </xsl:attribute>
</xsl:template>

<xsl:template match="write_cert">
  <xsl:attribute name="nodelabel">
    <xsl:value-of select="$ppacl2:write_cert"/>
  </xsl:attribute>
</xsl:template>

<!-- Demás -->

<xsl:template match="summary">
  <xsl:attribute name="nodelabel"><xsl:value-of select="$ppacl2:summary"/></xsl:attribute>
  <xsl:attribute name="leaf">1</xsl:attribute>
</xsl:template>

<xsl:template match="rule">
    <xsl:attribute name="hidden">1</xsl:attribute>
    <xsl:call-template name="search-dependency"/>
</xsl:template>

<xsl:template match="script">
  <xsl:attribute name="nodelabel" select="$ppacl2:proof"/>

  <!-- First put everything in a variable -->
  <xsl:variable name="new_element">
    <xsl:element name="dead_code">
      <xsl:for-each select="//*[@name and @package][not(ancestor::*[@name and @package]) and not(descendant::failure|descendant::error)]">
        <xsl:variable name="myName" select="@name"/>

        <!-- Dead code: any event which is not used for any proofs in its following siblings -->
        <xsl:if test="count(following-sibling::*[descendant::rule[@name = $myName]]) = 0">
          <xsl:element name="dead_event">
            <xsl:attribute name="name"      select="$myName"/>
            <xsl:attribute name="uid_event" select="generate-id(.)"/>
          </xsl:element>
        </xsl:if>
      </xsl:for-each>
    </xsl:element>
  </xsl:variable>

  <!-- Redump it, but with the UIDs. Ugly hack around the fact that
       generate-id() always generates the same ID for the same context
       node, so we need to use different context nodes to get
       different IDs. -->
  <xsl:for-each select="$new_element/*">
    <xsl:element name="dead_code">
      <xsl:attribute name="nodelabel" select="$ppacl2:deadcode"/>
      <xsl:attribute name="UID" select="generate-id()"/>
      <xsl:for-each select="*">
        <xsl:element name="dead_event">
          <xsl:attribute name="hidden" select="1"/>
          <xsl:copy-of select="@*"/>
        </xsl:element>
      </xsl:for-each>
    </xsl:element>
  </xsl:for-each>
</xsl:template>

<xsl:template match="*" priority="-2">
    <xsl:attribute name="hidden">1</xsl:attribute>
</xsl:template>

<!-- PLANTILLAS CON NOMBRE -->

<!-- Añade el icono de éxito/fracaso al evento -->
<xsl:template name="attach-success-icon">
    <xsl:choose>
      <xsl:when test="descendant::failure|descendant::error">
        <xsl:attribute name="nodeicon"><xsl:value-of select="$xmleye:failure-icon"/></xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="nodeicon"><xsl:value-of select="$xmleye:success-icon"/></xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<!-- Enlaza con las reglas que ha usado para su demostración -->
<xsl:template name="search-dependency">
    <xsl:variable name="rulename" select="@name"/>
    <xsl:for-each select="preceding::defun[@name = $rulename][position() = 1]
                          | preceding::defthm[@name = $rulename][position() = 1]
                          | preceding::thm[@name = $rulename][position() = 1]">
      <xsl:call-template name="create-local-link"/>
    </xsl:for-each>
    <xsl:for-each select="preceding::event[text() = $rulename][position() = 1]">
      <xsl:call-template name="create-external-link">
	<xsl:with-param name="rulename" select="$rulename"/>
      </xsl:call-template>
    </xsl:for-each>
</xsl:template>

<!-- Enlazada con las reglas que la usan para sus demostraciones -->
<xsl:template name="search-uses">
  <xsl:element name="usedby">
    <!-- Nodo oculto no seleccionable -->
    <xsl:attribute name="hidden">1</xsl:attribute>
    <xsl:variable name="rulename" select="@name"/>    

    <xsl:for-each select="following::defun[descendant::rule[@name = $rulename]]
                          | following::defthm[descendant::rule[@name = $rulename]]
			  | following::thm[descendant::rule[@name = $rulename]]">
      <xsl:element name="use">
        <xsl:call-template name="create-local-link"/>
      </xsl:element>
    </xsl:for-each>
 
  </xsl:element>
</xsl:template>

<!-- Crea un atributo href con un enlace al nodo actual -->
<xsl:template name="create-local-link">
   <xsl:attribute name="href">
     <xsl:value-of select="generate-id()"/>
   </xsl:attribute>
</xsl:template>

<!-- Crea un atributo href con un enlace al documento que contiene
     dicho evento.  

     Primero consultamos el eje ancestor::dep para ver todos los
     elementos padre. Lo colocamos en una variable para que al
     consultarla sea en orden de documento (y no en orden inverso).

     Ver http://www.stylusstudio.com/xsllist/200305/post50970.html. -->
<xsl:template name="create-external-link">
  <xsl:param name="rulename"/>

  <!-- Separamos en paquete y nombre -->
  <xsl:variable name="package" select="substring-before($rulename, '::')"/>
  <xsl:variable name="name"    select="substring-after($rulename, '::')"/>
  <xsl:variable name="dirs"    select="ancestor::dep/@book_dir"/>
  <xsl:variable name="fich"    select="ancestor::dep/@book_file[1]"/>

  <!-- Caracteres especiales -->
  <xsl:variable name="quot">'</xsl:variable>

  <xsl:attribute name="href">
    <xsl:value-of select="concat($dirs, $fich, '#xpointer(//*[@name=', 
                                 $quot, $name, $quot, ' and @package=', 
                                 $quot, $package, $quot, '])')"/>
  </xsl:attribute>
</xsl:template>

</xsl:stylesheet>
